@@ -1,4 +1,6 @@
-/* -*- mode: c; c-basic-offset: 4; -*- */
+/* -*- tab-width:8 c-basic-offset:4  indent-tabs-mode:nil -*- */
+/* set vi: ts=8:softtabstop=4,shiftwidth=4,expandtab */
+
 /*
  *  Data acquisition daemon access
  */
@@ -50,7 +52,7 @@ struct daq_private_;
      int rc = daq_startup();
      if (rc) {
          printf("global initialization failed\n");
-	 return 1;
+         return 1;
      }
 
      -- Connect to server
@@ -61,16 +63,16 @@ struct daq_private_;
      rc = daq_connect(&daqd, server, port, nds_v2);
      if (rc) {
          printf("Connection failed with error: %s\n", daq_strerror(rc));
-	 return 2;
+         return 2;
      }
 
      -- Specify channels
      --
      const char* channel[4] = {
          "H1:DMT-STRAIN", 
-	 "L1:DMT-STRAIN", 
-	 "V1:h16384Hz",
-	 0
+         "L1:DMT-STRAIN", 
+         "V1:h16384Hz",
+         0
      }
 
      int i;
@@ -86,7 +88,7 @@ struct daq_private_;
      rc = daq_request_data(&daqd, start_gps, end_gps, delta);
      if (rc) {
          printf("Data request failed with error: %s\n", daq_strerror(rc));
-	 return 3;
+         return 3;
      }
 
      --  Read data blocks
@@ -94,16 +96,16 @@ struct daq_private_;
      time_t t;
      for (t=start_gps; t<end_gps; t+=delta) {
          rc = daq_recv_next(&daqd);
-	 if (rc) {
+         if (rc) {
              printf("Receive data failed with error: %s\n", daq_strerror(rc));
-	     return 4;
-	 }
+             return 4;
+         }
 
-	 --  Get data
+         --  Get data
          --
-	 chan_req_t* stat = daq_get_channel_status(&daqd, channel[1]);
-	 if (!stat || stat->status <= 0) break;
-	 
+         chan_req_t* stat = daq_get_channel_status(&daqd, channel[1]);
+         if (!stat || stat->status <= 0) break;
+         
      }
 
      --  Disconnect from server
@@ -262,7 +264,7 @@ struct daq_ {
     size_t tb_size;
 
     /**  signal conversion data; dynamically allocated and reallocated 
-      *	 as needed; freed in daq_recv_shutdown()
+      *         as needed; freed in daq_recv_shutdown()
       */
     struct signal_conv1 *s;
 
@@ -289,7 +291,7 @@ struct daq_ {
 
     /** Handle to internal data
      */
-    struct daq_private_*	conceal;
+    struct daq_private_*        conceal;
 };
 
 typedef struct daq_ daq_t;
@@ -486,7 +488,7 @@ daq_init(daq_t *daq);
   */
 DLL_EXPORT void
 daq_init_channel(daq_channel_t* chan, const char* name, enum chantype ctype,
-		 double rate, daq_data_t dtype);
+                 double rate, daq_data_t dtype);
 
 /**  Test whether the socket is connected.
   *  \brief Test for connected server.
@@ -562,7 +564,7 @@ daq_recv_shutdown (daq_t* daq);
   */
 DLL_EXPORT int 
 daq_recv_channels (daq_t* daq, daq_channel_t* channel, int num_channels, 
-		   int* num_channels_received);
+                   int* num_channels_received);
 
 /**  Get a channel list hash for the specified channel type and gps time. The
   *  \a gps and \a type parameters are interpreted exactly as for the
@@ -582,7 +584,29 @@ daq_recv_channels (daq_t* daq, daq_channel_t* channel, int num_channels,
   */
 DLL_EXPORT int 
 daq_recv_channel_hash(daq_t* daq, void* hash, int* length, 
-		      time_t gps, enum chantype type);
+                      time_t gps, enum chantype type);
+
+/**  Get a channel list hash for the specified channel type and gps time and
+  *  channel name pattern. The \a gps, \a type and pattern parameters are 
+  *  interpreted exactly as for the daq_recv_channels_by_pattern() function. 
+  *  The preliminary hash code implementation is a crc-32 so the length of 
+  *  the hash buffer must be at least 4 bytes. If the hashing scheme changes 
+  *  a longer buffer may be required. This function is implemented only for 
+  *  nds2 clients. 
+  *  \brief Get a channel list hash.
+  *  \param daq    Pointer to client status structure.
+  *  \param hash   Pointer to a buffer to receive the hash code.
+  *  \param length Pointer to integer that on entry contains the length of
+  *                the \a hash buffer and on return contains the length of
+  *                the receved hash code. 
+  *  \param gps    validity time (or zero for current channels)
+  *  \param type   Limit hash to include only channels of the specified type.
+  *  \param pat    Channel name pattern.
+  *  \return DAQD status code.
+  */
+DLL_EXPORT int 
+daq_recv_hash_by_pattern(daq_t* daq, void* hash, int* length, 
+                         time_t gps, enum chantype type, const char* pat);
 
 /**  Get a list of channel names and information. A list of all channels 
   *  available at the specified time and of the specified type is returned.
@@ -613,7 +637,60 @@ daq_recv_channel_hash(daq_t* daq, void* hash, int* length,
   */
 DLL_EXPORT int 
 daq_recv_channel_list(daq_t* daq, daq_channel_t* channel, int num_channels, 
-		   int* num_channels_received, time_t gps, enum chantype type);
+                   int* num_channels_received, time_t gps, enum chantype type);
+
+/**  Get a list of channel names and information. A list of all channels 
+  *  matching the specified pattern available at the specified time and of 
+  *  the specified type is returned. A \c GPS time of zero indicates currently 
+  *  available channels. A type code of \c cUnknown (0) gives all channel types
+  *  available at the specified gps time. The pattern bash-like channel name
+  *  template used to select the channels to be matched. A null string will 
+  *  match all channel names.. Note that this function
+  *  may produce a list with multiple channel entries with identical names.
+  *  The number of channels returned in
+  *  \c *num_channels_received is the total number of channels available from
+  *  the server and may exceed the number of channels for which space has been
+  *  allocated. In fact, an efficient way to find out how much space is needed
+  *  for the channel list is to call daq_recv_channel_list with \c num_channels
+  *  set to zero and the channel list pointer set to NULL.
+  *  \remarks
+  *  This function only works completely with the NDS2 server. If a non-zero
+  *  GPS or a data type other than \c cOnline is requested, the function 
+  *  returns \c DAQD_VERSION_MISMATCH and \c *num_channels_received is set 
+  *  to zero. Wild-card pattern matching is not currently implemented on the
+  *  server, althoug an emty string will match all channel names and a 
+  *  non-empty will match achannel with exactly the specified name.
+  *  \brief Get a list of channels.
+  *  \param daq Pointer to client status structure.
+  *  \param channel List of daq_channel structures to receive information.
+  *  \param num_channels Number of channel structures allocated.
+  *  \param num_channels_received Pointer to integer to receive number of 
+  *                               channels defined on server (may exceed 
+  *                               \c num_channels ).
+  *  \param gps validity time (or zero for current channels)
+  *  \param type List only channels of the specified type.
+  *  \param pat  Channel list pattern.
+  *  \return DAQD status code.
+  */
+DLL_EXPORT int 
+daq_recv_channels_by_pattern(daq_t* daq, daq_channel_t* channel, 
+                             int num_channels, int* num_channels_received, 
+                             time_t gps, enum chantype type, const char* pat);
+
+/**  Get a list of epoch names and their gps time ranges. The list is 
+  *  returned into a user defined character buffer of spcified length. 
+  *  The number of bytes of data received is returned in a integer length
+  *  variable specified by the user. The epoch list is a string with blank 
+  *  separated field of the form \c "\<epoch-name>=\<gps-start>-\<gps-stop>".
+  *  \brief Get a list of epochs.
+  *  \param daq     Pointer to client status structure.
+  *  \param epochs  Character buffer to receive the epoch definition list.
+  *  \param max_len Length (in bytes) of the allocated buffer.
+  *  \param str_len Pointer to int to be filled with string length in bytes.
+  *  \return DAQD status code.
+  */
+DLL_EXPORT int 
+daq_recv_epoch_list(daq_t* daq, char* epochs, size_t max_len, int* str_len);
 
 /**  Receive an eight-digit hex ID.
   *  \brief Receive an ID.
@@ -637,7 +714,7 @@ daq_recv_id (daq_t *daq);
   */
 DLL_EXPORT int 
 daq_recv_source_data(daq_t* daq, char* sources, size_t max_len, 
-		     time_t gps, long* str_len);
+                     time_t gps, long* str_len);
 
 /**  Get a list of channel sources for the requested channels at the specified
   *  gps time. A \c GPS time of zero indicates currently available channels. 
@@ -653,7 +730,7 @@ daq_recv_source_data(daq_t* daq, char* sources, size_t max_len,
   */
 DLL_EXPORT int 
 daq_recv_source_list(daq_t* daq, char* sources, size_t max_len, 
-		     time_t gps, long* str_len);
+                     time_t gps, long* str_len);
 
 /**  Add the specified channel to the request list. If the type is set to 
   *  the default value (\c cUnknown or 0), the channel type will be inferred 
@@ -669,7 +746,7 @@ daq_recv_source_list(daq_t* daq, char* sources, size_t max_len,
   */
 DLL_EXPORT int
 daq_request_channel(daq_t* daq, const char* name, enum chantype type,
-		    double rate);
+                    double rate);
 
 /**  Add the specified channel to the request list. The channel name data
   *  type, sample rate, etc are copied from the channel list entry.
