@@ -37,26 +37,6 @@
 #include "nds2.h"
 #include "trench.h"
 
-/***************************************************************************
- *                                                                         *
- *  Note that this code was moved here from daq_internal.h because it      *
- *  uses macros that are only defined during the library compilation       *
- *  and was therefore preventing external packages from compiling against  *
- *  nds2-client. This code should probably find a permanent home in a      *
- *  truely internal header, in daq_config.h or some similar location.      *
- *                                                                         *
- ***************************************************************************/
-
-#if SIZEOF_SIZE_T == SIZEOF_INT
-#define PRISIZE_T "u"
-#elif SIZEOF_SIZE_T == SIZEOF_LONG
-#define PRISIZE_T "lu"
-#elif SIZEOF_SIZE_T == SIZEOF_LONG_LONG
-#define PRISIZE_T "llu"
-#else
-#error Unable to figure out how to print size_t
-#endif
-
 /**************************************************************************/
 
 #define TRACE_ENTRY_EXIT 0
@@ -347,6 +327,7 @@ typedef size_t write_len_t;
         }
         else if (resp < 0)
         {
+            resp = DAQD_ERROR;
             nds_logging_printf( "ERROR: %s: Server error: %s\n",
                                 METHOD,
                                 daq_strerror(resp) );
@@ -492,7 +473,7 @@ daq_recv_block (daq_t *daq) {
  
     /* read data samples */
     oread = read_bytes(fd, ptb->data, block_len);
-    if (!oread) {
+    if (oread < 0) {
         daq->err_num = DAQD_ERROR;
         return -1;
     }
@@ -568,9 +549,10 @@ daq_recv_next (daq_t *daq) {
     if (dt != 1) {
         fast_blen_divisor = 1;
     } else {
+        size_t fast_blen = 0;
+
         fast_blen_divisor = FAST_BLOCKS_PER_SEC;
         /* Calculate the block length for a 1/16 sec block */
-        size_t fast_blen = 0;
         for (i=0; i<N; ++i) {
             if (daq->chan_req_list[i].status >= 0) {
                 daq_data_t typ = daq->chan_req_list[i].data_type;
