@@ -1,4 +1,6 @@
-/* -*- mode: c; c-basic-offset: 4; -*- */
+/* -*- tab-width:8 c-basic-offset:4  indent-tabs-mode:nil -*- */
+/* set vi: ts=8:softtabstop=4,shiftwidth=4,expandtab */
+
 #ifndef DAQC_NDS1_H
 #define DAQC_NDS1_H
 
@@ -29,7 +31,20 @@ nds1_initalize(void);
  */
 int
 nds1_recv_channels (daq_t *daq, daq_channel_t *channel, int num_channels, 
-		    int *num_channels_received);
+                    int *num_channels_received);
+
+/*
+ *  Receive channel data using the NDS1 protocol for a single channel, by name.
+ *
+ *  FIXME: The NDS1 protocol's "status channels" command accepts any number of
+ *  channel names as an optional argument, but the metadata is only valid for
+ *  the zeroth channel. For the first, second, ..., Nth channels, the metadata
+ *  (sample rate, type, units, etc.) are blank. If this bug in the NDS1 servers
+ *  is ever fixed, then we should implement a multi-channel version of this
+ *  query.
+ */
+int
+nds1_recv_channel (daq_t *daq, daq_channel_t *channel, const char *name);
 
 /*  nds1_request_data(daq_t*, time_t, time_t)
  *
