@@ -5,6 +5,13 @@
 #include "daq_config.h"
 #endif /* HAVE_CONFIG_H */
 
+#if HAVE_WIN_H
+#include <win.h>
+#endif /* HAVE_WIN_H */
+#if HAVE_STDARG_H
+#include <stdarg.h>
+#endif
+
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
@@ -45,6 +52,42 @@
 #define NFDS(x) (x+1)
 #endif
 
+/*
+ * Pulled from the swig client
+ */
+static int 
+nds_str_endswith(const char *str, const char *end)
+{
+	const size_t str_len = strlen(str);
+	const size_t end_len = strlen(end);
+
+	return end_len <= str_len
+		&& memcmp(str + (str_len - end_len), end, end_len) == 0;
+}
+
+static ssize_t
+nds_str_lastindex(const char *str, int ch)
+{
+	const char *cur = str;
+    char _ch = (char)ch;
+	ssize_t index = 0;
+	if (!str || ch == '\0')
+	{
+		return -1;
+	}
+	while (*cur)
+	{
+		cur++;
+		index++;
+	}
+	while (index >= 0 && *cur != _ch)
+	{
+		cur--;
+		index--;
+	}
+	return index;
+}
+
 /*
  *  Connect to the DAQD server on the host identified by `ip' address.
  *  Returns zero if OK or the error code if failed.
@@ -247,6 +290,233 @@ cvthex(const char** s, char* buf, void* x) {
     *(int*)x = (int)( dca_strtol(buf) );
 }
 
+/*
+ * Some trend types have differing data types than the base channel type.  (Ex: ".mean", ".rms"
+ * return double precision values for integer channels)
+ * 
+ * @param trend_name - The trend name (with the m-trend or s-trend removed) as a NULL terminated string
+ * @param base_channel_type - The type of the base data channel
+ * @return - The data type to use for the trend
+ */
+static daq_data_t 
+nds1_get_trend_data_type(const char *trend_name, daq_data_t base_channel_type)
+{
+    daq_data_t result = base_channel_type;
+    if (trend_name)
+    {
+        if (nds_str_endswith(trend_name, ".mean") || nds_str_endswith(trend_name, ".rms"))
+        {
+            result = _64bit_double;
+        }
+        else if (nds_str_endswith(trend_name, ".n"))
+        {
+            result = _32bit_integer;
+        }
+    }
+    return result;
+}
+
+static void
+_nl_to_null(char *s)
+{
+    char ch = 0;
+
+    if (!s) return;
+    for (ch = *s; ch != '\0' && ch != '\n'; ++ch) {}
+    if (ch == '\n')
+        *s = '\0';
+}
+
+static int
+_nds1_read_sc3_string(nds_socket_type sock, size_t max_len, char *buf)
+{
+    if (!buf || _daq_read_cstring( sock, max_len, buf) < 0)
+    {
+        return DAQD_ERROR;
+    }
+    _nl_to_null(buf);
+    return DAQD_OK; 
+}
+
+static int
+_nds1_read_sc3_number(nds_socket_type sock, size_t max_len, char *buf, long *out)
+{
+    char *endp = 0;
+    long tmp = 0;
+
+    if (_nds1_read_sc3_string( sock, max_len, buf ) != DAQD_OK)
+    {
+        return DAQD_ERROR;
+    }
+    tmp = strtol(buf, &endp, 10);
+    if (*endp != '\0')
+    {
+        perror("Error channel meta-data");
+        return DAQD_ERROR;   
+    }
+    *out = tmp;
+    return DAQD_OK;
+}
+
+static int
+_nds1_read_sc3_double(nds_socket_type sock, size_t max_len, char *buf, double *out)
+{
+    char *endp = 0;
+    double tmp = 0.0;
+
+    if (_nds1_read_sc3_string( sock, max_len, buf ) != DAQD_OK)
+    {
+        return DAQD_ERROR;
+    }
+    tmp = strtod(buf, &endp);
+    if (*endp != '\0')
+    {
+        perror("Error channel meta-data");
+        return DAQD_ERROR;   
+    }
+    *out = tmp;
+    return DAQD_OK;
+}
+
+static int
+nds1_recv_channels_sc3 (daq_t *daq, daq_channel_t *channel, int num_channels,
+                        int *num_channels_received, const char *name) {
+#if _WIN32
+typedef int write_len_t;
+#else /* _WIN32 */
+typedef size_t write_len_t;
+#endif /* _WIN32 */
+
+    static const char* METHOD = "nds1_recv_channels_sc3";
+#if _WIN32
+#define BUF_SIZE (MAX_LONG_CHANNEL_NAME_LENGTH + 2)
+#else /* _WIN32 */
+    const int BUF_SIZE = MAX_LONG_CHANNEL_NAME_LENGTH + 2;
+#endif /* _WIN32 */
+    char chan_req[ sizeof("status channels 3 {\"\"};\n") 
+        + MAX_LONG_CHANNEL_NAME_LENGTH ];
+    int channel_name_size = MAX_LONG_CHANNEL_NAME_LENGTH + 2;
+    char buf[ BUF_SIZE ];
+    const char *tmp = 0;
+    long numeric_val = 0;
+    double double_val = 0.0;
+    const int signal_unit_size = 40;
+    int result = DAQD_ERROR;
+
+    /* for right now just done named entries on 12.1+ */
+    if (!name || ((daq->nds1_ver == 12 && daq->nds1_rev == 0) || daq->nds1_ver < 12 ) )    
+        return DAQD_NOT_SUPPORTED;
+
+    if ( strlen(name) > MAX_LONG_CHANNEL_NAME_LENGTH )
+        return DAQD_INVALID_CHANNEL_NAME;
+
+#if HAVE_SPRINTF_S
+    sprintf_s
+#else /* HAVE_SPRINTF_S */
+    snprintf
+#endif /* HAVE_SPRINTF_S */
+        ( chan_req, sizeof(chan_req),
+          "status channels 3 {\"%s\"};\n",
+          name );
+    
+    if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
+                            10 ) )
+    {
+        nds_logging_printf( "INFO: %s: nds1_recv_channels_sc3: entry\n",
+                            METHOD );
+    }
+
+    /** Cannot use daq_send as status channels 3 { ... } does not send
+     * a response code on success, only error.
+     */
+    if ( write (daq->conceal->sockfd, chan_req, (write_len_t) strlen(chan_req)) != strlen(chan_req) )
+    {
+        perror("Error in write()");
+        return DAQD_WRITE;
+    }
+    /* On success the output from the server is the channel name, not 0000.
+     * So read in the first byte, if it is a '0' we have an error, else
+     * read the rest of the channel name
+     */
+    if ( read_bytes( daq->conceal->sockfd, buf, (size_t)1 ) != 1 ) 
+    {
+        perror("Error in read_bytes()");
+        return DAQD_ERROR;
+    }
+    /* Error condition */
+    if ( buf[0] == '0' ) {
+        if ( read_bytes( daq->conceal->sockfd, buf + 1, (size_t)3 ) != 3 ) {
+            perror("Error in read_bytes()");
+            return DAQD_ERROR;
+        }
+        buf[4] = '\0';
+        return dca_strtol(buf);
+    }
+    /* At this point buf has the first character of the name, read in the rest */
+    if (_nds1_read_sc3_string( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH+2-1, buf+1) != DAQD_OK)
+        goto clean_up;
+    tmp = buf;
+    _daq_get_string( &tmp, buf+MAX_LONG_CHANNEL_NAME_LENGTH+2, channel->name, sizeof(channel->name) );
+    /* rate */
+    if (_nds1_read_sc3_number( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &numeric_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->rate = (double)numeric_val;
+    /* data type */
+    if (_nds1_read_sc3_number( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &numeric_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->data_type = (daq_data_t)numeric_val;
+    channel->bps = (int)data_type_size(channel->data_type);
+    /* TP num */
+    if (_nds1_read_sc3_number( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &numeric_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->tpnum = (daq_data_t)numeric_val;
+    /* group */
+    if (_nds1_read_sc3_number( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &numeric_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->type = cvt_group_chantype((int)numeric_val);
+    /* Fast channels have non-zero group numbers, cvt_group_chantype maps them
+     * to cUnknown.  Map them to cOnline.
+     */
+    if (channel->type == cUnknown)
+        channel->type = cOnline;
+    /* units */
+    if (_nds1_read_sc3_string( daq->conceal->sockfd, (size_t)signal_unit_size, buf) != DAQD_OK)
+        goto clean_up;
+
+    tmp = buf;
+    _daq_get_string( &tmp, buf+signal_unit_size+2, channel->s.signal_units, sizeof(channel->s.signal_units));
+    /* gain */
+    if (_nds1_read_sc3_double( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &double_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->s.signal_gain = (float)double_val;
+    /* slope */
+    if (_nds1_read_sc3_double( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &double_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->s.signal_slope = (float)double_val;
+    /* offset */
+    if (_nds1_read_sc3_double( daq->conceal->sockfd, (size_t)MAX_LONG_CHANNEL_NAME_LENGTH, buf, &double_val) != DAQD_OK)
+        goto clean_up;
+
+    channel->s.signal_offset = (float)double_val;
+    result = DAQD_OK;
+
+    *num_channels_received = 1;
+clean_up:
+    if (result != DAQD_OK)
+        perror("Unable to read channel meta-data");
+
+    return result;
+#if defined(BUF_SIZE)
+#undef BUF_SIZE
+#endif /* DEFINED(BUF_SIZE) */
+}
+
 static int
 nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
                          int *num_channels_received, const char *name) {
@@ -255,6 +525,7 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
     int resp;
     size_t lChanWd = 8;
     int    channels = -1;
+    int spurious_bytes = 0;     /* NDS1 < v12.1 with a named channel adds a spurious \t */
 #if _WIN32
     nds_socket_type f;
 #else
@@ -278,9 +549,30 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
         if ( strlen(name) > MAX_LONG_CHANNEL_NAME_LENGTH )
             return DAQD_INVALID_CHANNEL_NAME;
 
-        snprintf ( chan_req, sizeof(chan_req),
-                   "status channels 2 {\"%s\"};\n",
-                   name );
+#if HAVE_SPRINTF_S
+        sprintf_s
+#else /* HAVE_SPRINTF_S */
+        snprintf
+#endif /* HAVE_SPRINTF_S */
+            ( chan_req, sizeof(chan_req),
+              "status channels 2 {\"%s\"};\n",
+              name );
+        /* When dealing w/ status channels 2 { name(s) };
+         * NDS1 < v12.1 have an extra tab between the name and the meta data
+         * NDS1 >= v12.1 get the byte ordering wrong on some fields
+         */
+        if ( (daq->nds1_ver == 12 && daq->nds1_rev >= 1) || daq->nds1_ver > 12 ) {
+            /* status channels 2 { "named channel" }; is completely broken on 12.1.  Padding is incorrectly
+             * applied to 3 of the fields, so a sample rate of 16Hz and 16kHz are encoded as 10000000 amd
+             * 40000000 respectively.  The same thing applies to the test point num, type, and group.  
+             * So we must use status channels 3 { "named channel "}; when we are on 12.1.
+             * Might as well use it 12.1+ as well.
+             */
+            return nds1_recv_channels_sc3(daq, channel, num_channels, num_channels_received, name);
+        }
+        if (( daq->nds1_ver == 12 && daq->nds1_rev == 0 ) || daq->nds1_ver < 12) {
+            spurious_bytes = 1;
+        }
     }
 
     /* set channel name, signal unit size based on protocol - K. Thorne*/
@@ -302,8 +594,7 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
 
     /* FIXME: hack for stray tab in "status channels 2" output
      * for named channels from broken servers */
-    if (name)
-        chan_buf_size++;
+    chan_buf_size += spurious_bytes;
 
     if ( nds_logging_check( NDS_LOG_GROUP_VERBOSE_ERRORS,
                             10 ) )
@@ -379,8 +670,7 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
             s = buf + channel_name_size;
             /* FIXME: hack for stray tab in "status channels 2" output
              * for named channels from broken servers */
-            if (name)
-                s++;
+            s += spurious_bytes;
 
             /* Read rate                                    */
             cvthex(&s, buf, &intval);
@@ -392,6 +682,11 @@ nds1_recv_channels_impl (daq_t *daq, daq_channel_t *channel, int num_channels,
             /* Group number and data type are 16-bit fields */
             cvthex(&s, buf, &intval);
             channel[i].type = cvt_group_chantype(intval >> 16);
+            /* Fast channels have non-zero group numbers, cvt_group_chantype maps them
+             * to cUnknown.  Map them to cOnline.
+             */
+            if (channel[i].type == cUnknown)
+                channel[i].type = cOnline;
             channel[i].data_type = (intval & 0xffff);
             channel[i].bps= (int)data_type_size(channel[i].data_type);
 
@@ -435,6 +730,9 @@ nds1_recv_channels (daq_t *daq, daq_channel_t *channel, int num_channels,
 /*
  *  Receive channel data using the NDS1 protocol for a single channel, by name.
  *
+ *  Note: NDS1 does not understand trend channel names (H1:SOME_CHANNEL.min,s-trend).
+ *  This function does the neccesary work to handle trends with NDS1 servers.
+ *
  *  FIXME: The NDS1 protocol's "status channels" command accepts any number of
  *  channel names as an optional argument, but the metadata is only valid for
  *  the zeroth channel. For the first, second, ..., Nth channels, the metadata
@@ -445,7 +743,67 @@ nds1_recv_channels (daq_t *daq, daq_channel_t *channel, int num_channels,
 int
 nds1_recv_channel (daq_t *daq, daq_channel_t *channel, const char *name) {
     int received;
-    return nds1_recv_channels_impl(daq, channel, 1, &received, name);
+    int result = 0;
+    const char *channel_name = name;
+    chantype_t channel_type = cRaw;
+    double sample_rate;
+    int is_trend = 0;
+    char trend_name[MAX_LONG_CHANNEL_NAME_LENGTH+1];
+    char base_name[MAX_LONG_CHANNEL_NAME_LENGTH+1];
+
+    if (!name || !channel || !daq)
+    {
+	return DAQD_ERROR;
+    } 
+    if (nds_str_endswith(name, ",m-trend"))
+    {
+	channel_type = cMTrend;
+	sample_rate = 1.0/60;
+	is_trend = 1;
+    }
+    else if (nds_str_endswith(name, ",s-trend"))
+    {
+	channel_type = cSTrend;
+	sample_rate = 1.0;
+	is_trend = 1;
+    }
+
+    if (is_trend)
+    {
+	ssize_t index;
+
+	memset(trend_name, 0, (size_t)(MAX_LONG_CHANNEL_NAME_LENGTH+1));
+	memset(base_name, 0, (size_t)(MAX_LONG_CHANNEL_NAME_LENGTH+1));
+
+	strncpy(trend_name, name, strlen(name)-strlen(",m-trend"));
+	index = nds_str_lastindex(trend_name, '.');
+	if (index >= 0)
+	{
+	    strncpy(base_name, trend_name, (size_t)index);
+	}
+	else
+	{
+	    strncpy(base_name, trend_name, (size_t)(MAX_LONG_CHANNEL_NAME_LENGTH+1));
+	} 
+	is_trend = 1;
+	channel_name = base_name;
+    }
+
+    result = nds1_recv_channels_impl(daq, channel, 1, &received, channel_name);
+    if (result == 0 && is_trend)
+    {
+	/* We need to use the trend name to request data.  Patch in name, 
+	 * the sample rate, and type information gathered above.
+	 */
+	strncpy(channel->name, trend_name, (size_t)(MAX_LONG_CHANNEL_NAME_LENGTH+1));
+	/* for trends the data type of the trend may differ from the base channel type
+	 * ex, an int channel will have a double type as the mean trend
+	 */
+	channel->data_type = nds1_get_trend_data_type(trend_name, channel->data_type);
+	channel->type = channel_type;
+	channel->rate = sample_rate;
+    }
+    return result;
 }
 
 /*  nds1_request_data(daq_t*, time_t, time_t)
@@ -652,7 +1010,7 @@ nds1_receive_reconfigure(daq_t* daq, size_t block_len) {
             if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
                                     0 ) )
             {
-                nds_logging_printf( "malloc(%zi) failed; errno=%d\n", 
+                nds_logging_printf( "malloc(%" PRISIZE_T ") failed; errno=%d\n", 
                                     alloc_size, errno);
             }
             daq -> s_size = 0;
@@ -666,7 +1024,7 @@ nds1_receive_reconfigure(daq_t* daq, size_t block_len) {
             if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
                                     0 ) )
             {
-                nds_logging_printf ( "realloc(%zi) failed; errno=%d\n",
+                nds_logging_printf ( "realloc(%" PRISIZE_T ") failed; errno=%d\n",
                                      alloc_size, errno );
             }
             daq->s_size = 0;
