@@ -30,6 +30,8 @@
 
 #define NEW_VECT(type,dim) ((type*)malloc(dim*sizeof(type)))
 
+#define NDS_EXIT_STATUS_FAIL 99 /* A general failure has occurred */
+
 enum pgm_mode {
     list_group,
     chan_list,
@@ -70,7 +72,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	dtype = chan->data_type;
 
 	N = (size_t)chan->status / data_type_size(dtype);
-	printf("Channel: %s  type: %s  nWords: %zi\n", 
+	printf("Channel: %s  type: %s  nWords: %" PRISIZE_T "\n", 
 	       chan->name, data_type_name(dtype), N);
 
 	switch (dtype) {
@@ -82,7 +84,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    p = (short*)( daq_get_block_data( daq )
 			  + chan->offset);
 	    for (j=0; j<N; j+=spl) {
-		printf("%5zi", j);
+		printf("%5" PRISIZE_T, j);
 		for (k=j; k<j+spl && k<N; ++k) {
 		    printf(" ");
 		    printf(sfmt, (int)*p++);
@@ -98,7 +100,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    p = (int32_t*)(daq_get_block_data( daq )
 			   + chan->offset);
 	    for (j=0; j<N; j+=spl) {
-		printf("%5zi", j);
+		printf("%5" PRISIZE_T, j);
 		for (k=j; k<j+spl && k<N; ++k) {
 		    printf(" ");
 		    printf(ifmt, (int)*p++);
@@ -114,7 +116,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    p = (float*)(daq_get_block_data( daq )
 			 + chan->offset);
 	    for (j=0; j<N; j+=spl) {
-		printf("%5zi", j);
+		printf("%5" PRISIZE_T, j);
 		for (k=j; k<j+spl && k<N; ++k) {
 		    printf(" ");
 		    printf(ffmt, *p++);
@@ -130,7 +132,7 @@ dump_data(daq_t* daq, size_t spl, const char* fmt) {
 	    p = (double*)(daq_get_block_data( daq )
 			  + chan->offset);
 	    for (j=0; j<N; j+=spl) {
-		printf("%5zi", j);
+		printf("%5" PRISIZE_T, j);
 		for (k=j; k<j+spl && k<N; ++k) { 
 		    printf(" ");
 		    printf(dfmt, *p++);
@@ -157,7 +159,7 @@ dump_status(daq_t* daq) {
     printf("Data for %i seconds starting at GPS: %i\n", 
 	   daq_get_block_secs( daq ),
 	   daq_get_block_gps( daq ) );
-    printf("%-40s %-7s %s\n", "Channel", "type", "nWords");
+    printf("%-40s %-7s   %-8s %s\n", "Channel", "type", "nWords", "units");
     for (i=0; i<daq->num_chan_request; i++) {
 	chan_req_t* chan = daq->chan_req_list + i;
 	daq_data_t dtype = chan->data_type;
@@ -167,8 +169,8 @@ dump_status(daq_t* daq) {
 		   daq_strerror(-chan->status));
 	} else {
 	    size_t N = (size_t) chan->status / data_type_size(dtype);
-	    printf("%-40s %-7s %zi\n", chan->name, 
-		   data_type_name(chan->data_type), N);
+	    printf("%-40s %-7s %8" PRISIZE_T " %s\n", chan->name, 
+		   data_type_name(chan->data_type), N, chan->s.signal_units);
 	}
     }
 }
@@ -184,6 +186,8 @@ main(int argc, const char* argv[]) {
     enum pgm_mode mode   = chan_data;
     enum chantype c_type = cUnknown;
 
+    int retval = 0;
+
     short port_id   = DAQD_PORT;
     const char* cmd = 0;
     const char* node_id = "localhost";
@@ -434,7 +438,9 @@ main(int argc, const char* argv[]) {
     /*----------------------------------  request epoch                   */
     if (epoch != NULL) {
 	rc = daq_set_epoch(&daq, epoch);
-	if (rc) printf("error in set_epoch: %s\n", daq_strerror(rc));
+	if (rc) {
+            printf("error in set_epoch: %s\n", daq_strerror(rc));
+        }
     }
 
 
@@ -510,8 +516,10 @@ main(int argc, const char* argv[]) {
  	} else {
             rc = daq_recv_channel_hash(&daq, &hash, &len, gps_start, c_type);
 	}
-	if (rc)
+	if (rc) {
 	    printf("Error reading channel hash: %s\n", daq_strerror(rc));
+            retval = NDS_EXIT_STATUS_FAIL;
+        }
 	else
 	    printf("Channel hash code: %x\n", hash);
 	break;
@@ -535,6 +543,7 @@ main(int argc, const char* argv[]) {
 	buf[ MAX_EPOCH_LIST - 1 ] = 0;
 	if (rc) {
 	    printf("Error reading epoch list: %s\n", daq_strerror(rc));
+            retval = NDS_EXIT_STATUS_FAIL;
 	}
 	else {
 	    int ich, last=0;
@@ -597,6 +606,7 @@ main(int argc, const char* argv[]) {
  	    }
 	    if (rc) {
 		printf("Error reading channel count: %s\n", daq_strerror(rc));
+                retval = NDS_EXIT_STATUS_FAIL;
 		break;
 	    }
 	    if (mode == chan_count) {
@@ -629,6 +639,7 @@ main(int argc, const char* argv[]) {
  	    }
 	    if (rc) {
 		printf("Error reading channel list: %s\n", daq_strerror(rc));
+                retval = NDS_EXIT_STATUS_FAIL;
 		free(channel_list);
 		break;
 	    }
@@ -744,6 +755,7 @@ main(int argc, const char* argv[]) {
 	    rc = daq_request_data(&daq, gps_start, gps_end, delta);
 	    if (rc) {
 		printf("Error in daq_request_data: %s\n", daq_strerror(rc));
+                retval = NDS_EXIT_STATUS_FAIL;
 	    } else {
 		time_t gps, dt=delta;
 		for (gps=gps_start; gps<gps_end; gps+=dt) {
@@ -751,6 +763,7 @@ main(int argc, const char* argv[]) {
 		    if (rc) {
 			printf("Error in daq_recv_next: %s\n", 
 			       daq_strerror(rc));
+                        retval = NDS_EXIT_STATUS_FAIL;
 			break;
 		    }
 		    dump_status(&daq);
@@ -786,5 +799,6 @@ main(int argc, const char* argv[]) {
 			    __FILE__ );
     }
     daq_recv_shutdown(&daq);
-    return 0;
+
+    return retval;
 }
