@@ -326,7 +326,8 @@ nds2_recv_hash_impl(daq_t *daq, void *hash, int *length, time_t gps,
     const char* hash_cmd_template="get-channel-crc XgpsgpsgpsX XunknownX {};";
     char* buf;
     size_t buf_len, pat_len;
-    int resp, fd;
+    int resp;
+    nds_socket_type fd;
 
     if (*length < sizeof(uint4_type)) return DAQD_ERROR;
 
@@ -593,7 +594,8 @@ int
 nds2_recv_epoch_list(daq_t *daq, char *epochs, size_t max_len, int* str_len) {
     const char* METHOD = "nds2_recv_epoch_list";
     const char* cmd = "list-epochs;";
-    int resp, fd, tmp_len;
+    int resp, tmp_len;
+    nds_socket_type fd;
 
     if (str_len) *str_len = 0;
     if (max_len == 0) return DAQD_ERROR;
@@ -906,7 +908,7 @@ nds2_receive_reconfigure(daq_t* daq, size_t block_len) {
         if ( nds_logging_check( NDS_LOG_GROUP_STATUS_UPDATE,
                                 0 ) )
         {
-            nds_logging_printf( "Channel reconfigure block length has bad length (%zi)\n",
+            nds_logging_printf( "Channel reconfigure block length has bad length (%" PRISIZE_T ")\n",
                                 block_len );
         }
     }
@@ -957,9 +959,9 @@ static int
 get_version_and_revision( daq_t* daq )
 {
     int retval;
-    int        v = UNSPECIFIED_NDS2_PROTOCOL_VERSION;
-    int        rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
-    int exp_rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
+    uint4_type v = UNSPECIFIED_NDS2_PROTOCOL_VERSION;
+    uint4_type rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
+    uint4_type exp_rv = UNSPECIFIED_NDS2_PROTOCOL_REVISION;
 
     /*-------------------------------------------------------------------
      * Obtain the server protocol version
