@@ -23,9 +23,14 @@
 #endif /* HAVE_CONFIG_H */
 
 #include "nds.h"
+
 #include <daqc.h>
 #include <daqc_response.h>
 #include <channel.h>
+
+#if HAVE_IO_H
+#include <io.h>
+#endif /* HAVE_IO_H */
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/stat.h>
@@ -44,6 +49,11 @@
 #ifdef _WIN32
 #include <Shlobj.h>
 #define snprintf _snprintf
+#ifndef R_OK
+#define R_OK 4
+#define W_OK 2
+#define F_OK 0
+#endif /* R_OK */
 #else
 #include <unistd.h>
 #include <pwd.h>
@@ -289,6 +299,9 @@ static nds2_data_type _nds2_data_to_data_type(daq_data_t c)
 		case _32bit_complex:
 			data_type = NDS2_DATA_TYPE_COMPLEX32;
 			break;
+		case _32bit_uint:
+			data_type = NDS2_DATA_TYPE_UINT32;
+			break;
 		default:
 			data_type = 0;
 	}
@@ -321,6 +334,9 @@ static daq_data_t _nds2_data_to_data_type_reverse(nds2_data_type c)
 		case NDS2_DATA_TYPE_COMPLEX32:
 			data_type = _32bit_complex;
 			break;
+		case NDS2_DATA_TYPE_UINT32:
+			data_type = _32bit_uint;
+			break;
 		default:
 			data_type = _undefined;
 	}
@@ -373,7 +389,7 @@ static int _nds2_open_db(nds2_connection *connection, const char *db_dir, const
 	 * removed when we can depend on newer sqlite versions */
 	int readonly = 0;
 
-	/* db path, e.g.
+	/* db path, e.g. */
 	/* "DB_DIR/0.10_nds.ligo.caltech.edu_31200.sqlite". */
 	if (asprintf(&db_path,
 		     "%s/" PACKAGE_VERSION "_%s_%d.sqlite",
@@ -416,7 +432,7 @@ static int _nds2_open_db(nds2_connection *connection, const char *db_dir, const
 	return ret;
 }
 
-/* FIXME: use the following once we can depend on sqlite > 3.7.11
+/* FIXME: use the following once we can depend on sqlite > 3.7.11 */
 /**
  * Return readonly status of db: 0=read/write, 1=readonly
  *
@@ -458,7 +474,7 @@ static int _nds2_find_db(nds2_connection *connection, const int create)
 
 	if (env_dir) {
 
-		if (asprintf(&db_dir, env_dir) < 0)
+		if (asprintf(&db_dir, "%s", env_dir) < 0)
 			goto done;
 
 		ret = _nds2_open_db(connection, db_dir, create);
@@ -1011,10 +1027,16 @@ static nds2_channel **_nds2_db_find_channels(
 			}
 			if (data_type_mask & (NDS2_DATA_TYPE_INT32 | NDS2_DATA_TYPE_FLOAT32))
 			{
-				if (strjoin(&sql, "SELECT name || '.min,s-trend' AS _name, 8 AS _channel_type, (CASE WHEN data_type <= 2 THEN 2 ELSE 8 END) AS _data_type, 1 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, "SELECT name || '.min,s-trend' AS _name, 8 AS _channel_type, (CASE WHEN data_type == 32 THEN 8 ELSE data_type END) AS _data_type, 1 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
 				if (strjoin(&sql, name_cond, " ")) goto fail;
 				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
-				if (strjoin(&sql, "SELECT name || '.max,s-trend' AS _name, 8 AS _channel_type, (CASE WHEN data_type <= 2 THEN 2 ELSE 8 END) AS _data_type, 1 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, "SELECT name || '.max,s-trend' AS _name, 8 AS _channel_type, (CASE WHEN data_type == 32 THEN 8 ELSE data_type END) AS _data_type, 1 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, name_cond, " ")) goto fail;
+				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
+			}
+			if (data_type_mask & NDS2_DATA_TYPE_INT32)
+			{
+				if (strjoin(&sql, "SELECT name || '.n,s-trend' AS _name, 8 AS _channel_type, 2 AS _data_type, 1 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
 				if (strjoin(&sql, name_cond, " ")) goto fail;
 				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
 			}
@@ -1030,10 +1052,16 @@ static nds2_channel **_nds2_db_find_channels(
 			}
 			if (data_type_mask & (NDS2_DATA_TYPE_INT32 | NDS2_DATA_TYPE_FLOAT32))
 			{
-				if (strjoin(&sql, "SELECT name || '.min,m-trend' AS _name, 16 AS _channel_type, (CASE WHEN data_type <= 2 THEN 2 ELSE 8 END) AS _data_type, 1.0/60 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, "SELECT name || '.min,m-trend' AS _name, 16 AS _channel_type, (CASE WHEN data_type == 32 THEN 8 ELSE data_type END) AS _data_type, 1.0/60 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
 				if (strjoin(&sql, name_cond, " ")) goto fail;
 				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
-				if (strjoin(&sql, "SELECT name || '.max,m-trend' AS _name, 16 AS _channel_type, (CASE WHEN data_type <= 2 THEN 2 ELSE 8 END) AS _data_type, 1.0/60 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, "SELECT name || '.max,m-trend' AS _name, 16 AS _channel_type, (CASE WHEN data_type == 32 THEN 8 ELSE data_type END) AS _data_type, 1.0/60 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
+				if (strjoin(&sql, name_cond, " ")) goto fail;
+				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
+			}
+			if (data_type_mask & NDS2_DATA_TYPE_INT32)
+			{
+				if (strjoin(&sql, "SELECT name || '.n,m-trend' AS _name, 16 AS _channel_type, 2 AS _data_type, 1.0/60 AS _sample_rate, signal_gain, signal_slope, signal_offset, signal_units FROM channels WHERE", " UNION ALL ")) goto fail;
 				if (strjoin(&sql, name_cond, " ")) goto fail;
 				if (strjoin(&sql, "data_type & ?3", " and ")) goto fail;
 			}
@@ -1061,8 +1089,6 @@ static nds2_channel **_nds2_db_find_channels(
 	while ((retval = sqlite3_step(stmt)) == SQLITE_ROW)
 	{
 		nds2_channel *channel;
-		const char *name;
-		const char *signal_units;
 
 		if (count_channels_found >= count_channels_allocated)
 		{
@@ -1075,14 +1101,14 @@ static nds2_channel **_nds2_db_find_channels(
 		}
 
 		channel = nds2_channel_alloc(
-			sqlite3_column_text(stmt, 0),
+			(const char*)sqlite3_column_text(stmt, 0),
 			sqlite3_column_int(stmt, 1),
 			sqlite3_column_int(stmt, 2),
 			sqlite3_column_double(stmt, 3),
 			(float)sqlite3_column_double(stmt, 4),
 			(float)sqlite3_column_double(stmt, 5),
 			(float)sqlite3_column_double(stmt, 6),
-			sqlite3_column_text(stmt, 7));
+			(const char*)sqlite3_column_text(stmt, 7));
 
 		if (!channel)
 			goto fail;
@@ -1389,6 +1415,7 @@ int nds2_sizeof_data_type(nds2_data_type data_type)
 			break;
 		case NDS2_DATA_TYPE_INT32:
 		case NDS2_DATA_TYPE_FLOAT32:
+		case NDS2_DATA_TYPE_UINT32:
 			ret = 4;
 			break;
 		case NDS2_DATA_TYPE_INT64:
@@ -1568,7 +1595,7 @@ int nds2_iterate(
 {
 	int ret = -1;
 	int result;
-	size_t i, j;
+	size_t i;
 	int have_minute_trends = 0;
 	double bps = 0;
 
@@ -1595,8 +1622,6 @@ int nds2_iterate(
 		daq_channel_t channel;
 		chantype_t channel_type;
 		double sample_rate;
-		daq_data_t data_type;
-		int data_size;
 
 		memset(&channel, 0, sizeof(channel));
 
@@ -1674,12 +1699,12 @@ int nds2_iterate(
 				/* start with smallest stride that is a multiple of 60 seconds and
 				 * whose frame size in bytes is greater than or equal to
 				 * _NDS2_TYPICAL_BYTES_PER_FRAME. */
-				stride = 60 * ceil(_NDS2_TYPICAL_BYTES_PER_FRAME / (60 * bps));
+				stride = (long)(60 * ceil(_NDS2_TYPICAL_BYTES_PER_FRAME / (60 * bps)));
 			} else { /* request does not include minute trends */
 				/* start with smallest stride that is a multiple of 1 second and
 				 * whose frame size in bytes is greater than or equal to
 				 * _NDS2_TYPICAL_BYTES_PER_FRAME. */
-				stride = ceil(_NDS2_TYPICAL_BYTES_PER_FRAME / bps);
+				stride = (long)(ceil(_NDS2_TYPICAL_BYTES_PER_FRAME / bps));
 			}
 
 			if (stride > gps_stop - gps_start) /* stride is longer than request duration */
